@@ -29,14 +29,9 @@
 /* Init function */
 void       cscm_init                          (void);
 
-/* Scheme to C functions... */
-void       add_slot                           (SCM        slot_data);
-GList*     new_deck                           (SCM        deck_data);
-hcard_type new_card                           (SCM        card_data);
-gboolean   has_options                        (void);
-
 /* C to Scheme functions... */
-SCM        make_card                          (hcard_type card);
+SCM        c2scm_card                          (hcard_type card);
+gboolean   cscmi_has_options                        (void);
 SCM        cscmi_start_game_lambda            (void);
 SCM        cscmi_button_pressed_lambda        (SCM        slot_id,
 					       SCM        cards);
@@ -53,6 +48,6 @@ SCM        cscmi_winning_game_lambda          (void);
 SCM        cscmi_hint_lambda                  (void);
 SCM        cscmi_get_options_lambda           (void);
 SCM        cscmi_apply_options_lambda         (SCM        options);
-SCM        cscmi_timeout_lambda               (void);
+gboolean   cscmi_timeout_lambda               (void);
 
 #endif
