@@ -43,11 +43,30 @@ GtkAction *redoaction;
 GtkAction *restartaction;
 GtkAction *fullscreenaction;
 GtkAction *leavefullscreenaction;
+GtkAction *menuaction = NULL;
 GtkWidget *helpitem;
 GtkWidget *menubar;
 GtkWidget *toolbar;
 
 static gchar * gamename = NULL;
+static gchar * ugamename = NULL;
+
+static gchar *convert_name_to_underscored (const gchar *inname)
+{
+  gchar *outname;
+  gchar *s;
+
+  outname = g_strdup (inname);
+
+  s = outname;
+  while (*s) {
+    if (*s == ' ')
+      *s = '_';
+    s++;
+  }
+
+  return outname;
+}
 
 static void restart_game ()
 {
@@ -55,10 +74,11 @@ static void restart_game ()
   /* Treat a restart as part of the same game. Eventually either
    * the player will win or loose and then it gets counted. */
   game_in_progress = FALSE;
+  option_list_set_sensitive ();
   new_game (NULL, &seed);
 };
 
-void random_seed ()
+static void new_game_action ()
 {
   if (waiting_for_mouse_up()) return;
   new_game (NULL, NULL);
@@ -100,14 +120,25 @@ static void help_about_callback (void)
 	  NULL
   };
 
+  const gchar *documenters[] = {
+	  "Rosanna Yuen (zana@webwynk.net)",
+	  NULL
+  };
+
   gtk_show_about_dialog (GTK_WINDOW (app),
 			 "name", _("AisleRiot"),
 			 "version", VERSION,
-			 "comments", _("AisleRiot provides a rule-based solitaire card engine "
-				       "that allows many different games to be played."),
-			 "copyright", "Copyright \xc2\xa9 1998-2005 Jonathan Blandford",
+			 "comments", _("AisleRiot provides a rule-based solitaire\n"
+				       "card engine that allows many different\n"
+				       "games to be played."),
+			 "copyright", "Copyright \xc2\xa9 1998-2006 Jonathan Blandford",
+			 "license", "GPL 2+",
 			 "authors", authors,
+			 "documenters", documenters,
 			 "translator_credits", _("translator-credits"),
+			 "logo-icon-name", "gnome-aisleriot.png",
+			 "website", "http://www.gnome.org/projects/gnome-games/",
+			 "wrap-license", TRUE,
 			 NULL);
 }
 
@@ -118,7 +149,7 @@ static void general_help (void)
 
 static void help_on_specific_game (void)
 {
-  gnome_help_display ("aisleriot.xml", gamename, NULL);
+  gnome_help_display ("aisleriot.xml", ugamename, NULL);
 }
 
 static void toolbar_toggle_callback(GtkToggleAction * togglebutton, 
@@ -195,16 +226,16 @@ void redo_set_sensitive (gboolean state)
   gtk_action_set_sensitive (redoaction, state);
 }
 
-const GtkActionEntry actions[] = {
+static const GtkActionEntry actions[] = {
   { "GameMenu", NULL, N_("_Game") },
   { "ViewMenu", NULL, N_("_View") },
   { "ControlMenu", NULL, N_("_Control") },
   { "OptionsMenu", NULL, "Options" /* not translated on purpose */ },
   { "HelpMenu", NULL, N_("_Help") },
 
-  { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (random_seed) },
+  { "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_game_action) },
   { "RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL, NULL, G_CALLBACK (restart_game) },
-  { "Select", GTK_STOCK_INDEX, N_("_Select Game..."), NULL, N_("Play a different game"), G_CALLBACK (show_select_game_dialog) },
+  { "Select", GTK_STOCK_INDEX, N_("_Select Game..."), "<Ctrl>o", N_("Play a different game"), G_CALLBACK (show_select_game_dialog) },
   { "Statistics", GTK_STOCK_ADD, N_("S_tatistics"), NULL, N_("Show gameplay statistics"), G_CALLBACK (show_statistics_dialog) },
   { "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_app) },
   { "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) },
@@ -215,16 +246,16 @@ const GtkActionEntry actions[] = {
   { "RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, NULL, G_CALLBACK (redo_callback) },
   { "Hint", GAMES_STOCK_HINT, NULL, NULL, NULL, G_CALLBACK (show_hint_dialog) },
   { "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, N_("View help for Aisleriot"), G_CALLBACK (general_help) },
-  { "Help", GTK_STOCK_HELP, NULL, NULL, N_("View help for this game"), G_CALLBACK (help_on_specific_game) },
+  { "Help", GTK_STOCK_HELP, NULL, "<Shift>F1", N_("View help for this game"), G_CALLBACK (help_on_specific_game) },
   { "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (help_about_callback) },
 };
 
-const GtkToggleActionEntry toggles[] = {
+static const GtkToggleActionEntry toggles[] = {
   { "Toolbar", NULL, N_("_Toolbar"), NULL, "Show or hide the toolbar", G_CALLBACK (toolbar_toggle_callback) },
   { "ClickToMove", NULL, N_("_Click to Move"), NULL, "Pick up and drop cards by clicking", G_CALLBACK (clickmove_toggle_callback) },
 };
 
-const char *ui_description = 
+static const char ui_description[] = 
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='GameMenu'>"
@@ -265,7 +296,6 @@ const char *ui_description =
 "    <toolitem action='RedoMove'/>"
 "    <toolitem action='Hint'/>" 
 "    <toolitem action='LeaveFullscreen'/>"
-/* "    <toolitem action='Hidden'/>" */
 "  </toolbar>"
 "</ui>";
 
@@ -325,7 +355,11 @@ void help_update_game_name (gchar * name)
   if (gamename)
     g_free (gamename);
 
+  if (ugamename)
+    g_free (ugamename);
+
   gamename = g_strdup (name);
+  ugamename = convert_name_to_underscored (name);
 
   gtk_label_set_text (GTK_LABEL (GTK_BIN (helpitem)->child), gamename);
 
@@ -334,20 +368,11 @@ void help_update_game_name (gchar * name)
 static gchar * make_option_gconf_key (void)
 {
   static gchar *basekey = "/apps/aisleriot/rules/";
-  gchar *name, *s, *r, *sk;
+  gchar *r, *sk;
   GConfSchema *schema;
   GConfValue *def;
-
-  name = g_strdup (gamename);
-  s = name;
-  while (*s) {
-    if (*s == ' ')
-      *s = '_';
-    s++;
-  }
     
-  r = g_strconcat (basekey, name, NULL);
-  g_free(name);
+  r = g_strconcat (basekey, ugamename, NULL);
 
   sk = g_strconcat ("/schemas", r, NULL);
 
@@ -381,8 +406,8 @@ static gint compress_options_to_int (SCM options_list)
   SCM entry;
 
   bits = 0;
-  l = SCM_INUM (scm_length (options_list));
-  for (i=0; i<l; i++) {
+  l = SCM_INUM (scm_length (options_list)) - 1;
+  for (i=l; i>=0; i--) {
     entry  = scm_list_ref (options_list, SCM_MAKINUM (i));
     bits <<= 1;
     bits |= SCM_NFALSEP (scm_list_ref (entry, SCM_MAKINUM (1))) ? 1 : 0;
@@ -408,6 +433,14 @@ static void expand_options_from_int (SCM options_list, guint bits)
 /* FIXME: There isn't a schema for these automatically generated keys.
  * Maybe we should automatically generate those too. */
 
+void option_list_set_sensitive (void)
+{
+  if (menuaction == NULL)
+    return;
+  
+  gtk_action_set_sensitive (menuaction, !game_in_progress);
+}
+
 static void save_option_list (SCM options_list)
 {
   gchar *keyname;
@@ -461,7 +494,6 @@ void install_options_menu (gchar *name)
 {
   static int merge_id = 0;
   static GtkActionGroup *options_group = NULL;
-  GtkAction *menuaction;
   SCM options_list;
   gint l, i;
 
@@ -475,9 +507,9 @@ void install_options_menu (gchar *name)
     options_group = NULL;
   }
 
-  if (has_options ()) {
+  if (cscmi_has_options ()) {
     menuaction = gtk_action_group_get_action (action_group, "OptionsMenu");
-    g_object_set (G_OBJECT (menuaction), "label", _(name), NULL);
+    g_object_set (G_OBJECT (menuaction), "label", name, NULL);
 
     options_group = gtk_action_group_new ("OptionsActions");
     gtk_ui_manager_insert_action_group (ui_manager, options_group, -1);
