@@ -33,7 +33,6 @@
 #include "draw.h"
 #include <gnome.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <glade/glade.h>
 #include "slot.h"
 #include "card.h"
@@ -69,14 +68,12 @@ gboolean         dont_save = FALSE; /* If the game is selected on the
                                      * This is essential for Freecell
                                      * emulation.*/
 
-gint32		 enabled_features;
 gboolean	 droppable_is_featured;
 gboolean	 score_is_hidden; 
 
 guint            score;
 guint            timeout;
 guint32          seed;
-guint            n_games;
 gchar            *game_file = "";
 gchar            *game_name;
 gboolean         game_in_progress = FALSE;
@@ -86,10 +83,6 @@ gboolean         click_to_move = FALSE;
 gchar            *variation = "";
 gchar            *gamesdir;
 
-guint            x_spacing = 5;
-guint            y_spacing = 15;
-double           x_expanded_offset = 0.21;
-double           y_expanded_offset = 0.21;
 GConfClient      *gconf_client = NULL;
 
 #define DEFAULT_VARIATION "klondike.scm"
@@ -98,6 +91,7 @@ GConfClient      *gconf_client = NULL;
 gchar* game_file_to_name (const gchar* file)
 {
   char* p, *buf = g_path_get_basename(file);
+  gchar *ts;
 
   if ((p = strrchr (buf, '.'))) *p = '\0';
   for (p = buf; p = strchr(p, '_'), p && *p;) *p = ' ';
@@ -110,9 +104,15 @@ gchar* game_file_to_name (const gchar* file)
   }
   buf[0] = g_ascii_toupper (buf[0]);
 
-  return buf;
+  ts = g_strdup (_(buf));
+  g_free (buf);
+
+  return ts;
 }
 
+/* Note that this is not the inverse of game_file_to_name. This
+ * only works on untranslated names. game_file_to_name only produces
+ * translated names. Be careful. */
 static gchar * game_name_to_file (const gchar *name)
 {
   char *p, *s;
@@ -160,11 +160,10 @@ void eval_installed_file (char *file)
     char *message = g_strdup_printf (
          _("Aisleriot can't load the file: \n%s\n\n"
            "Please check your Aisleriot installation"), installed_filename);
-    GtkWidget *w = gtk_message_dialog_new (GTK_WINDOW(app),
-                                       GTK_DIALOG_DESTROY_WITH_PARENT,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_OK,
-                                       message);
+    GtkWidget *w = gtk_message_dialog_new (NULL, 0,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   message);
 
     gtk_dialog_run (GTK_DIALOG(w));
     gtk_widget_destroy(w);
@@ -186,6 +185,36 @@ save_state (GnomeClient *client)
   return TRUE;
 }
 
+/* Check for the existence of a particular variation and fall back to
+   a default if it isn't found. */
+/* FIXME: There is a lot of duplication with the eval_installed_file
+   function, but we need this called earlier. */
+static void check_game_file_name (void)
+{
+  gchar *fullpath;
+  gchar *partialpath;
+  GtkWidget *dialog;
+
+  partialpath = g_strconcat (GAMESDIR, game_file, NULL);
+  fullpath = gnome_program_locate_file (NULL,
+					GNOME_FILE_DOMAIN_APP_DATADIR,
+					partialpath,
+					FALSE, NULL);
+  if (!g_file_test (fullpath, G_FILE_TEST_EXISTS)) {
+    dialog = gtk_message_dialog_new (NULL, 0,
+				     GTK_MESSAGE_INFO,
+				     GTK_BUTTONS_OK,
+				     _("Aisleriot cannot find the last game you played."));
+ gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), _("This usually occurs when you run an older version of Aisleriot which does not have the game you last played. The default game, Klondike, is being started instead."));
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+    game_file = DEFAULT_VARIATION;
+  }
+
+  g_free (fullpath);
+  g_free (partialpath);
+}
+
 void new_game (gchar* file, guint *seedp)
 {
   SCM size;
@@ -201,19 +230,17 @@ void new_game (gchar* file, guint *seedp)
 
   if (file && strcmp (file, game_file)) {
     game_file = file;
-
-    /* Explicitly drop any dragged cards. This shouldn't happen, but it
-     * has in the past so we should be defensive. See bug 121762. */
-    /* FIXME: we haven't actually fixed the cause of this. */
-    free_press_data ();
     
     /* Although this line slows down game switching by a noticeable amount, we
      * add it here in order to make sure all the original functions are
      * "clean". */
     eval_installed_file ("sol.scm");
-    eval_installed_file (file);
+    /* This is here so that the previous lines can catch bad installations 
+       and report it before reporting the fallback to defaults. */
+    check_game_file_name ();
+    eval_installed_file (game_file);
 
-    game_name = game_file_to_name (file);
+    game_name = game_file_to_name (game_file);
     update_statistics_display ();
     help_update_game_name (game_name);
     install_options_menu (game_name);
@@ -236,8 +263,7 @@ void new_game (gchar* file, guint *seedp)
   }
 
   g_random_set_seed(seed);
-  score = 0;
-  set_score();
+  set_score(0);
   timer_reset ();
 
   size = cscmi_start_game_lambda ();
@@ -264,21 +290,30 @@ void new_game (gchar* file, guint *seedp)
 
 GtkWidget *score_value;
 
-void set_score () 
+guint score;
+
+void set_score(guint new_score) 
 {
   char b [10];
+  score = new_score;
   g_snprintf (b, sizeof (b), "%6d  ", score);
-
   gtk_label_set_text(GTK_LABEL(score_value), b);
 }
 
+guint get_score()
+{
+  return score;
+}
+
 GtkWidget *time_value;
 
 static gint timer_cb ()
 {
   timeout = 3600;
-  cscmi_timeout_lambda ();
-  end_of_game_test();
+  /* All the games return #f and nothing else with this call, but just in
+   * case someone changes their mind in the future. */
+  if (cscmi_timeout_lambda ())
+    end_of_game_test();
   return 0;	
 }
 
@@ -347,14 +382,16 @@ static void create_sol_board ()
 
   slot_gc = gdk_gc_new (playing_area->window);
 
-  g_signal_connect (GTK_OBJECT(playing_area),"button_release_event",
+  g_signal_connect (G_OBJECT(playing_area),"button_release_event",
 		      GTK_SIGNAL_FUNC (button_release_event), NULL);
-  g_signal_connect (GTK_OBJECT (playing_area), "motion_notify_event",
+  g_signal_connect (G_OBJECT (playing_area), "motion_notify_event",
 		      GTK_SIGNAL_FUNC (motion_notify_event), NULL);
-  g_signal_connect (GTK_OBJECT (playing_area), "button_press_event",
+  g_signal_connect (G_OBJECT (playing_area), "button_press_event",
 		      GTK_SIGNAL_FUNC (button_press_event), NULL);
-  g_signal_connect (GTK_OBJECT (playing_area), "configure_event",
+  g_signal_connect (G_OBJECT (playing_area), "configure_event",
 		      GTK_SIGNAL_FUNC (configure_event), NULL);
+  g_signal_connect (G_OBJECT (playing_area), "expose_event",
+		    GTK_SIGNAL_FUNC (expose_event), NULL);
 }
 
 void quit_app (GtkMenuItem *menuitem)
@@ -447,6 +484,8 @@ static void main_prog(void *closure, int argc, char *argv[])
 
   gtk_main ();
 
+  gnome_accelerators_sync ();
+
   free_pixmaps();
   g_object_unref (surface);
 }
@@ -458,31 +497,35 @@ retrieve_state (GnomeClient *client)
                                        "/apps/aisleriot/game_file",
                                        "klondike.scm");
   card_style = games_gconf_get_string (gconf_client,
-				       "/apps/aisleriot/card_style",
-				       "bonded-new.png");
+				       THEME_GCONF_KEY,
+				       "bonded.svg");
 }
 
 int main (int argc, char *argv [])
 {
-  struct poptOption aisleriot_opts[] = {
-    {"variation", '\0', POPT_ARG_STRING, NULL, 0, NULL, NULL},
-    {NULL, '\0', 0, NULL, 0, NULL, NULL}
+  static const GOptionEntry aisleriot_opts[] = {
+    {"variation", 'v', 0, G_OPTION_ARG_STRING, &variation,
+     N_("Select the game to play"), N_("NAME")},
+    {NULL}
   };
   gchar * var_file;
-
-  aisleriot_opts[0].arg = &variation;
-  aisleriot_opts[0].descrip = N_("Variation on game rules");
-  aisleriot_opts[0].argDescrip = N_("NAME");
+  GOptionContext *option_context;
+  GnomeProgram *program;
 
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gnome_program_init ("aisleriot", VERSION,
- 		      LIBGNOMEUI_MODULE, 
- 		      argc, argv,
- 		      GNOME_PARAM_POPT_TABLE, aisleriot_opts,
- 		      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+  option_context = g_option_context_new ("");
+  g_option_context_add_main_entries (option_context,
+                                     aisleriot_opts,
+                                     GETTEXT_PACKAGE);
+  
+  program = gnome_program_init ("aisleriot", VERSION,
+				LIBGNOMEUI_MODULE, 
+				argc, argv,
+				GNOME_PARAM_GOPTION_CONTEXT, option_context,
+				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
   glade_init ();
 
   gconf_client = gconf_client_get_default ();
@@ -494,7 +537,7 @@ int main (int argc, char *argv [])
 			   NULL, NULL, NULL);
   load_statistics ();
   
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-aisleriot.png");
+  gtk_window_set_default_icon_name ("gnome-aisleriot.png");
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "save_yourself",
 		      GTK_SIGNAL_FUNC (save_state), 
 		      (gpointer) g_path_get_basename(argv[0]));
@@ -518,5 +561,7 @@ int main (int argc, char *argv [])
   }
 
   scm_boot_guile (argc, argv, main_prog, NULL);
+
+  g_object_unref (program);
   return 0;
 }
