@@ -326,9 +326,13 @@ static void gtk_gridboard_repaint(GtkGridBoard * gridboard) {
 	  phase = gridboard->pixmaps[x][y];
 	  phase = (phase - 1.0)/30.0;
 	  gridboard->theme->draw_piece (cx, x, y, phase);
-	} else if (gridboard->selected[x][y]) {
+	} else if (gridboard->selected[x][y] == SELECTED_A) {
 	  gridboard->theme->draw_hilight (cx, x, y);
 	}
+	if (gridboard->selected[x][y] == SELECTED_B) {
+	  gridboard->theme->draw_secondary_hilight (cx, x, y, 
+						    gridboard->board[x][y]);
+	}
 	gridboard->changed[x][y] = FALSE;
       }
     }
@@ -367,7 +371,7 @@ static gint gtk_gridboard_flip_pixmaps(gpointer data)
         gint flipped_tiles;
 
 	if (!GTK_WIDGET_DRAWABLE (GTK_WIDGET (gridboard))) {
-	  return FALSE;
+	  return gridboard->animate;
 	}
 
 	flipped_tiles=0;
@@ -387,10 +391,6 @@ static gint gtk_gridboard_flip_pixmaps(gpointer data)
                                 gridboard->pixmaps[x][y]=pcepm;
                                 flipped_tiles++;
 				gridboard->changed[x][y] = TRUE;
-			} else if (!gridboard->animate) {
-				/* Disabled animations consist of the final frame.*/
-				gridboard->pixmaps[x][y] = pcepm;
-				gridboard->changed[x][y] = TRUE;
                         } else if (curpm < pcepm) {
                                 gridboard->pixmaps[x][y]++;
                                 flipped_tiles++;
@@ -406,7 +406,8 @@ static gint gtk_gridboard_flip_pixmaps(gpointer data)
 	if (flipped_tiles) 
 	  gtk_gridboard_paint (gridboard);
 
-        return TRUE;
+        /* destroy the timeout if animate is false */
+        return gridboard->animate;
 }
 
 /* helper functions */
@@ -483,11 +484,13 @@ void gtk_gridboard_set_animate(GtkGridBoard *gridboard, gboolean animate) {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
         gridboard->animate=animate;
-	if (gridboard->timeoutid)
-	  return;
-	gridboard->timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
-					     gtk_gridboard_flip_pixmaps,
-					     (gpointer) gridboard);
+        if (animate) {
+	  if (gridboard->timeoutid)
+	    return;
+	  gridboard->timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
+					       gtk_gridboard_flip_pixmaps,
+					       (gpointer) gridboard);
+        } else gridboard->timeoutid = 0;
 }
 
 void gtk_gridboard_set_visibility(GtkGridBoard *gridboard, gboolean visibility) {
@@ -530,6 +533,10 @@ void gtk_gridboard_set_piece(GtkGridBoard *gridboard, int x, int y,
 
         gridboard->board[x][y]=piece;
 	
+	if (!gridboard->animate) {
+	  gridboard->pixmaps[x][y] = get_pixmap_num (gridboard->board[x][y]);
+	  gridboard->changed[x][y] = TRUE;
+	}
 }
 
 int gtk_gridboard_get_piece(GtkGridBoard *gridboard, int x, int y) 
