@@ -829,6 +829,10 @@ static void gtk_minefield_multi_release (GtkMineField *mfield, guint x, guint y,
         gint n, nx, ny, i, c2;
 	guint lose = 0;
 
+	
+	if (c == 0) /* The release was outside the main area. */
+		return;
+
         mfield->multi_mode = 0;
 
         n = 0;
@@ -1036,7 +1040,6 @@ static gint gtk_minefield_button_release(GtkWidget *widget, GdkEventButton *even
 					0, NULL);
 		}
 		mfield->mines[mfield->cdown].down = 0;
-                mfield->cdown = -1;
 		mfield->action = NO_ACTION;
 		mfield->bdown[event->button-1] = 0;
 		gtk_mine_draw(mfield, mfield->cdownx, mfield->cdowny);
@@ -1146,7 +1149,7 @@ void gtk_minefield_set_size(GtkMineField *mfield, guint xsize, guint ysize)
 {
         g_return_if_fail(mfield != NULL);
         g_return_if_fail(GTK_IS_MINEFIELD(mfield));
-
+	
 	if ((mfield->xsize == xsize) && (mfield->ysize == ysize))
 		return;
 
