@@ -23,11 +23,9 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <sys/time.h>
 #include <string.h>
 
 #include "gnothello.h"
@@ -62,8 +60,7 @@ guint tiles_to_flip = 0;
 gint64 milliseconds_total = 0;
 gint64 milliseconds_current_start = 0;
 
-gint bcount;
-gint wcount;
+guint game_in_progress;
 
 gint8 pixmaps[8][8] = {{0,0,0,0,0,0,0,0},
 		      {0,0,0,0,0,0,0,0},
@@ -83,11 +80,13 @@ gint8 board[8][8] = {{0,0,0,0,0,0,0,0},
 		    {0,0,0,0,0,0,0,0},
 		    {0,0,0,0,0,0,0,0}};
 
-MoveHistory game[61];
-
-gint8 move_count = 0;
+guint whose_turn;
+gint8 move_count;
+gint bcount;
+gint wcount;
 
 extern guint flip_final_id;
+extern gint8 squares[64];
 
 int session_flag = 0;
 int session_xpos = -1;
@@ -101,11 +100,14 @@ GdkGC *gridGC[2] = { 0 };
 
 static void new_network_game_cb(GtkWidget *widget, gpointer data);
 
-static const struct poptOption options[] = {
-  {NULL, 'x', POPT_ARG_INT, &session_xpos, 0, NULL, NULL},
-  {NULL, 'y', POPT_ARG_INT, &session_ypos, 0, NULL, NULL},
-  {"server", 's', POPT_ARG_STRING, &game_server, 0, N_("Iagno server to use")},
-  {NULL, '\0', 0, NULL, 0}
+static const GOptionEntry options[] = {
+  {"x", 'x', 0, G_OPTION_ARG_INT, &session_xpos, N_("X location of window"), 
+   N_("X")},
+  {"y", 'y', 0, G_OPTION_ARG_INT, &session_ypos, N_("Y location of window"), 
+   N_("Y")},
+  {"server", 's', 0, G_OPTION_ARG_STRING, &game_server, 
+   N_("Iagno server to use"), N_("NAME")},
+  {NULL}
 };
 
 GnomeUIInfo game_menu[] = {
@@ -170,10 +172,9 @@ new_game_cb (GtkWidget *widget, gpointer data)
 void
 undo_move_cb (GtkWidget *widget, gpointer data)
 {
-	gint8 which_computer;
-	gint i, j;
+	gint8 which_computer, xy;
 
-	if ((black_computer_level && white_computer_level) || !move_count)
+	if ((black_computer_level && white_computer_level) || move_count == 4)
 		return;
 
 	if (flip_final_id) {
@@ -183,46 +184,27 @@ undo_move_cb (GtkWidget *widget, gpointer data)
 
 	game_in_progress = 1;
 
-	if (black_computer_level || white_computer_level) {
-		if (black_computer_level)
-			which_computer = BLACK_TURN;
-		else
-			which_computer = WHITE_TURN;
-		move_count--;
-		while (game[move_count].me == which_computer && move_count > 0) {
-			pixmaps[game[move_count].x][game[move_count].y] = 100;
-			move_count--;
-		}
-		pixmaps[game[move_count].x][game[move_count].y] = 100;
-		memcpy (board, game[move_count].board, sizeof (gint8) * 8 * 8);
-	} else {
-		move_count--;
-		memcpy (board, game[move_count].board, sizeof (gint8) * 8 * 8);
-		pixmaps[game[move_count].x][game[move_count].y] = 100;
+ 	which_computer = OTHER_PLAYER (whose_turn);
+ 	undo();
+ 	board_copy();
+ 	xy=squares[move_count];
+ 	pixmaps[xy%10-1][xy/10-1] = 100;
+	if ((((whose_turn == WHITE_TURN) && white_computer_level) ||
+	     ((whose_turn == BLACK_TURN) && black_computer_level))
+	    && (move_count > 4)) {
+ 		undo();
+ 		board_copy();
+ 		xy=squares[move_count];
+ 		pixmaps[xy%10-1][xy/10-1] = 100;
 	}
 
-	whose_turn = game[move_count].me;
-
 	if (whose_turn == WHITE_TURN)
 		gui_message (_("Light's move"));
 	else
 		gui_message (_("Dark's move"));
 
-	wcount = 0;
-	bcount = 0;
-
-	for (i = 0; i < 8; i++)
-		for (j = 0; j < 8; j++) {
-			if (board[i][j] == WHITE_TURN)
-				wcount++;
-			if (board[i][j] == BLACK_TURN)
-				bcount++;
-		}
-
 	gui_status ();
-
 	tiles_to_flip = 1;
-
 	check_computer_players ();
 }
 
@@ -261,13 +243,20 @@ about_cb (GtkWidget *widget, gpointer data)
 {
 	const gchar *authors[] = {"Ian Peters", NULL};
 
+	const gchar *documenters[] = {"Eric Baudais", NULL};
+
 	gtk_show_about_dialog (GTK_WINDOW (window),
 			       "name", _("Iagno"),
 			       "version", VERSION,
-			       "copyright", "Copyright \xc2\xa9 1998-2004 Ian Peters",
+			       "copyright", "Copyright \xc2\xa9 1998-2006 Ian Peters",
+			       "license", "GPL 2+",
 			       "comments",_("A disk flipping game derived from Reversi."),
-			       "authors", authors, 
+			       "authors", authors,
+			       "documenters", documenters, 
 			       "translator_credits", _("translator-credits"),
+			       "logo-icon-name", "gnome-iagno",
+			       "website", "http://www.gnome.org/projects/gnome-games/",
+			       "wrap-license", TRUE,
 			       NULL);
 }
 
@@ -441,7 +430,13 @@ flip_pixmaps (gpointer data)
 	
 	for (i = 0; i < 8; i++)
 		for (j = 0; j < 8; j++) {
-			if (pixmaps[i][j] == 100) {
+			/* This first case only happens when undoing the "final flip". */
+			if ((pixmaps[i][j] == 101) && (board[i][j] != 0)) {
+				pixmaps[i][j] = board[i][j];
+				gui_draw_pixmap (pixmaps[i][j], i, j);
+				flipped_tiles = 1;				
+			} else if ((pixmaps[i][j] == 100) 
+				   || ((pixmaps[i][j] != 101) && (board[i][j] == 0))) {
 				pixmaps[i][j] = 101;
 				gui_draw_pixmap (0, i, j);
 				flipped_tiles = 1;
@@ -543,7 +538,6 @@ clear_board (void)
 			board[i][j] = 0;
 
 	memcpy (pixmaps, board, sizeof (gint8) * 8 * 8);
-	memcpy (game[0].board, board, sizeof (gint8) * 8 * 8);
 
 	bcount = 0;
 	wcount = 0;
@@ -556,8 +550,7 @@ init_new_game (void)
 {
 	clear_board ();
 	game_in_progress = 1;
-	move_count = 0;
-	whose_turn = BLACK_TURN;
+	move_count = 4;
 
 	undo_set_sensitive (FALSE);
 	
@@ -568,9 +561,9 @@ init_new_game (void)
 
 	bcount = 2;
 	wcount = 2;
+	init();
 
 	memcpy (pixmaps, board, sizeof (gint8) * 8 * 8);
-	memcpy (game[0].board, board, sizeof (gint8) * 8 * 8);
 
 	redraw_board ();
 
@@ -764,7 +757,8 @@ int
 main (int argc, char **argv)
 {
 	GnomeClient *client;
-	struct timeval tv;
+	GnomeProgram *program;
+	GOptionContext *context;
 
 	gnome_score_init ("iagno");
 
@@ -772,15 +766,17 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gettimeofday (&tv, NULL);
-	srand (tv.tv_usec);
+        context = g_option_context_new ("");
+        g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
+	program = gnome_program_init ("iagno", VERSION,
+				      LIBGNOMEUI_MODULE,
+				      argc, argv,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
+				      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+
+	gtk_window_set_default_icon_name ("gnome-iagno.png");
 
-	gnome_program_init ("iagno", VERSION,
-			    LIBGNOMEUI_MODULE,
-			    argc, argv,
-			    GNOME_PARAM_POPT_TABLE, options,
-			    GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/iagno.png");
 	client= gnome_master_client ();
 
 	g_object_ref (G_OBJECT (client));
@@ -796,12 +792,12 @@ main (int argc, char **argv)
 
 	load_pixmaps ();
 
+	gtk_widget_show (window);
+
 	if (session_xpos >= 0 && session_ypos >= 0) {
 		gdk_window_move (window->window, session_xpos, session_ypos);
 	}
 
-	gtk_widget_show (window);
-
 	buffer_pixmap = gdk_pixmap_new (drawing_area->window,
 					BOARDWIDTH, BOARDHEIGHT, -1);
 
@@ -811,6 +807,8 @@ main (int argc, char **argv)
 
 	gtk_main ();
 
+	g_object_unref (program);
+
 	return 0;
 }
 
